MODULE HexToSet;
(* =========================================================================  
   Hex to / from SET Conversion Dialog                                                                
                                                                              
   Author : Chris Burrows                                                     
   Created: Mar 2010                                                                                    
   (c) 2010 CFB Software                                                               
   http://www.astrobe.com                                            
                                                                              
   ========================================================================= *)

IMPORT  
  Sys := "[mscorlib]System",
  Cpm := "[System]System.ComponentModel",
  Wfm := "[System.Windows.Forms]System.Windows.Forms",
  Drw := "[System.Drawing]System.Drawing",
  Data := "[System.Data]System.Data",
  FormUtils;

TYPE 
  HexToSetForm* = POINTER TO RECORD (FormUtils.XForm)
    labelHex: Wfm.Label;
    textHex*: Wfm.TextBox;
    labelSet: Wfm.Label;
    textSet*: Wfm.TextBox;
    btnConvert: Wfm.Button;
    components: Cpm.Container
  END;

(* ==================================================================== *)

PROCEDURE OutRange(rangeStart, rangeEnd: INTEGER; VAR s: Sys.String);
BEGIN
  s := s + Sys.Convert.ToString(rangeStart);
  IF rangeEnd = rangeStart + 1 THEN 
    s := s + ", " + Sys.Convert.ToString(rangeEnd)
  ELSIF rangeEnd > rangeStart THEN 
    s := s + ".." + Sys.Convert.ToString(rangeEnd)
  END
END OutRange;

(* ==================================================================== *)

PROCEDURE HexToInt*(IN str: ARRAY OF CHAR): INTEGER;
  VAR p, n: INTEGER;
    ch: CHAR;
BEGIN [UNCHECKED_ARITHMETIC]
  n := 0;
  p := 0;
  REPEAT 
    ch := CAP(str[p]);
    IF (ch >= '0') & (ch <= '9') THEN 
      n := 16 * n + ORD(ch) - ORD('0')
    ELSIF (ch >= 'A') & (ch <= 'F') THEN
      n := 16 * n + ORD(ch) - ORD('A') + 10
    END;
    INC(p)
  UNTIL ch = 0X;
  RETURN n
END HexToInt;

(* ==================================================================== *)

PROCEDURE (frm: HexToSetForm) HexToSet(), NEW;  
VAR
  i, rangeStart, rangeEnd, hex: INTEGER;  
  separator: BOOLEAN;
  s: SET;
  str: Sys.String;
BEGIN
  hex := HexToInt(frm.textHex.get_Text()); 
  s := BITS(hex);
  str := "{";
  separator := FALSE;
  rangeStart := -1;
  rangeEnd := -1;
  FOR i := 0 TO 31 DO
    IF i IN s THEN 
      IF rangeStart < 0 THEN 
        rangeStart := i
      ELSIF (i - rangeEnd) > 1 THEN 
        IF separator THEN str := str + ", " ELSE separator := TRUE END; 
        OutRange(rangeStart, rangeEnd, str);
        rangeStart := i 
      END;
      rangeEnd := i 
    END
  END;
  IF rangeStart >= 0 THEN 
    IF separator THEN str := str + (", ") END;
    OutRange(rangeStart, rangeEnd, str)
  END;
  str := str + ('}');
  frm.textSet.set_Text(str)
END HexToSet;
   
 
(* ==================================================================== *)

PROCEDURE (frm: HexToSetForm) btnConvertClick(sender: Sys.Object; e: Sys.EventArgs), NEW;
BEGIN
  frm.HexToSet()
END btnConvertClick;

(* ==================================================================== *)

PROCEDURE (frm: HexToSetForm) OnActivatedAction(sender: Sys.Object; e: Sys.EventArgs), NEW;
VAR
  b: BOOLEAN;
BEGIN
  b := frm.textHex.Focus()
END OnActivatedAction;

(* ==================================================================== *)

PROCEDURE (frm: HexToSetForm) InitializeComponent*(), NEW;
CONST
  labelWidth = 60;
  textWidth = 200;
  spacer = 12;
  col1 = spacer;
  col2 = col1 + labelWidth + spacer;
  col3 = col2 + textWidth + spacer;
VAR
  rowNo: INTEGER;
BEGIN
  NEW(frm.components);
  
  frm.SuspendLayout();

  rowNo := 1;
  frm.NewLabel(frm.labelHex, "Hex number:", col1, rowNo);
  frm.NewTextBox(frm.textHex, textWidth, col2, rowNo);
  
  frm.NewButton(frm.btnConvert, "&Convert", col3, rowNo * FormUtils.rowHeight);
  REGISTER(frm.btnConvert.Click, frm.btnConvertClick); 
  
  rowNo := 3;
  frm.NewLabel(frm.labelHex, "SET:", col1, rowNo);
  frm.NewTextBox(frm.textSet, textWidth, col2, rowNo);

  INC(rowNo, 2);
  frm.set_StartPosition(Wfm.FormStartPosition.CenterScreen);
  frm.set_Text("Convert Hex to SET");
  frm.set_AutoScaleBaseSize(Drw.Size.init(5, 13));
  frm.set_ClientSize(Drw.Size.init(col3 + frm.btnConvert.get_Width() + spacer, rowNo * FormUtils.rowHeight));
  frm.set_FormBorderStyle(Wfm.FormBorderStyle.FixedDialog);
  frm.set_MinimizeBox(FALSE);
  frm.set_MaximizeBox(FALSE);

  REGISTER(frm.Activated, frm.OnActivatedAction);
  frm.ResumeLayout(FALSE)
END InitializeComponent;

END HexToSet.
